#
# Copyright (C) 2023 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

""" Helper for building observation ingestion manifest """
import logging
import sys
from pathlib import Path
from typing import List

from ingest_envoy.manifest_components import (
    AncillaryProduct,
    OutputScienceProduct,
)
from ingest_envoy.utilities import AncillaryProductType, ScienceProductType

FITS = "fits"
IDIFITS = "idifits"
XML = "xml"
LOG = "log"
LOGGZ = "log.gz"
TEXT = "txt"
CSV = "csv"
TT0 = "tt0"
RMS = "rms"
PB = "pb"
MASK = "mask"
ALPHA = "alpha"
PBCOR = "pbcor"
BIN = "bin"


# pylint: disable=R1721
class ObservationIngestionProductsFinder:
    """Finds ancillary science products and other ancillary products needed for observation ingestion"""

    def __init__(self, files_found: list[Path], sp_type: ScienceProductType):
        self.logger = logging.getLogger("ingest_envoy")
        self.sp_type = sp_type
        self.files_found = files_found
        self.output_science_products = self._find_output_science_products()
        self.ancillary_products = self._find_ancillary_products()

    def _find_output_science_products(self) -> List[OutputScienceProduct]:
        """
        Find the ancillary products belonging to the science product*
        in the staging dir.

        N.B. ingestion artifacts file will not be there until manifest has been created!

        :return:
        """

        # Ingest all *fits files, or directories that contain XML files (assumed to be SDMs), as science products
        fits_files = [file for file in self.files_found if file.name.endswith(IDIFITS)]
        sdm_dirs = [file.parent for file in self.files_found if file.name == "ASDM.xml"]
        output_product_file_paths = fits_files + sdm_dirs
        self.logger.info(f"Science Product(s) to ingest: {output_product_file_paths}")

        science_products = []
        for file in output_product_file_paths:
            # Add all science products and their ancillaries
            science_products.append(
                OutputScienceProduct(
                    type=ScienceProductType.EXEC_BLOCK,
                    filename=file.name,
                )
            )

        return science_products

    def _find_ancillary_products(self) -> List[AncillaryProduct]:
        """
        Find the ancillary observation products in the staging dir: there should be a
        weblog and a pipeline artifacts tar. (The ingestion artifacts tar will be produced
        during the building of the manifest.)

        :return:
        """
        log_files = [file for file in self.files_found if file.name.endswith(LOG) or file.name.endswith(LOGGZ)]
        text_files = [file for file in self.files_found if file.name.endswith(TEXT)]
        self.logger.info(f"Ancillary product(s) to ingest: {[*log_files, *text_files]}")

        ancillary_products = []
        for file in log_files:
            ancillary_products.append(AncillaryProduct(type=AncillaryProductType.LOG_TYPE, filename=file.name))
        for file in text_files:
            ancillary_products.append(
                AncillaryProduct(type=AncillaryProductType.OBSERVATION_DETAILS, filename=file.name)
            )

        return ancillary_products
